@@ -98,7 +98,7 @@ int ipkg_install_from_file(ipkg_conf_t *conf, const char *filename)
      /* XXX: CLEANUP: hash_insert_pkg has a nasty side effect of possibly
 	freeing the pkg that we pass in. It might be nice to clean this up
 	if possible.  */
-     pkg = hash_insert_pkg(&conf->pkg_hash, pkg, 1);
+     pkg = hash_insert_pkg(&conf->pkg_hash, pkg, 1,conf);
      old = pkg_hash_fetch_installed_by_name(&conf->pkg_hash, pkg->name);
 
      pkg->local_filename = strdup(filename);
@@ -123,6 +123,7 @@ int ipkg_install_from_file(ipkg_conf_t *conf, const char *filename)
 	  }
      }
 
+     ipkg_message(conf, IPKG_DEBUG2,"Function: %s calling ipkg_install_pkg \n",__FUNCTION__);
      return ipkg_install_pkg(conf, pkg);
 }
 
@@ -177,6 +178,7 @@ ipkg_error_t ipkg_install_by_name(ipkg_conf_t *conf, const char *pkg_name)
 	etc.). This is a real mess and will need to be cleaned up if
 	anyone ever wants to make a nice libipkg. */
 
+     ipkg_message(conf, IPKG_DEBUG2,"Function: %s calling ipkg_install_pkg \n",__FUNCTION__);
      return ipkg_install_pkg(conf, new);
 }
 
@@ -184,15 +186,19 @@ ipkg_error_t ipkg_install_multi_by_name(ipkg_conf_t *conf, const char *pkg_name)
 {
      abstract_pkg_vec_t *providers = pkg_hash_fetch_all_installation_candidates (&conf->pkg_hash, pkg_name);
      int i;
+     ipkg_error_t err;
+     abstract_pkg_t *ppkg ;
 
      if (providers == NULL)
 	  return IPKG_PKG_HAS_NO_CANDIDATE;
 
      for (i = 0; i < providers->len; i++) {
-	  abstract_pkg_t *ppkg = abstract_pkg_vec_get(providers, i);
-	  ipkg_error_t err = ipkg_install_by_name(conf, ppkg->name);
+	  ppkg = abstract_pkg_vec_get(providers, i);
+          ipkg_message(conf, IPKG_DEBUG2,"Function: %s calling ipkg_install_by_name %d \n",__FUNCTION__, i);
+	  err = ipkg_install_by_name(conf, ppkg->name);
 	  if (err)
 	       return err;
+/* XXX Maybe ppkg should be freed ? */
      }
      return 0;
 }
@@ -232,6 +238,7 @@ int pkg_mark_dependencies_for_installation(ipkg_conf_t *conf, pkg_t *pkg, pkg_ve
      }
 
      if (ndepends <= 0) {
+	  pkg_vec_free(depends);
 	  return 0;
      }
 
@@ -366,6 +373,7 @@ int satisfy_dependencies_for(ipkg_conf_t *conf, pkg_t *pkg)
 	     it in, so check first. */
 	  if ((dep->state_status != SS_INSTALLED)
 	      && (dep->state_status != SS_UNPACKED)) {
+               ipkg_message(conf, IPKG_DEBUG2,"Function: %s calling ipkg_install_pkg \n",__FUNCTION__);
 	       err = ipkg_install_pkg(conf, dep);
 	       if (err) {
 		    pkg_vec_free(depends);
@@ -609,12 +617,13 @@ int pkg_remove_installed_replacees(ipkg_conf_t *conf, pkg_vec_t *replacees)
 /* to unwind the removal: make sure they are installed */
 int pkg_remove_installed_replacees_unwind(ipkg_conf_t *conf, pkg_vec_t *replacees)
 {
-     int i;
+     int i, err;
      int replaces_count = replacees->len;
      for (i = 0; i < replaces_count; i++) {
 	  pkg_t *replacee = replacees->pkgs[i];
 	  if (replacee->state_status != SS_INSTALLED) {
-	       int err = ipkg_install_pkg(conf, replacee);
+               ipkg_message(conf, IPKG_DEBUG2,"Function: %s calling ipkg_install_pkg \n",__FUNCTION__);
+	       err = ipkg_install_pkg(conf, replacee);
 	       if (err)
 		    return err;
 	  }
@@ -670,6 +679,7 @@ static int ipkg_install_check_downgrade(ipkg_conf_t *conf, pkg_t *pkg, pkg_t *ol
 	  ipkg_message(conf, IPKG_NOTICE,
 		       "Installing %s (%s) to %s...\n",
 		       pkg->name, version, pkg->dest->name);
+	  free(version);
 	  return 0;
      }
 }
@@ -688,6 +698,9 @@ int ipkg_install_pkg(ipkg_conf_t *conf, pkg_t *pkg)
 		       "INTERNAL ERROR: null pkg passed to ipkg_install_pkg\n");
 	  return -EINVAL;
      }
+
+     ipkg_message(conf, IPKG_DEBUG2, "Function: %s calling pkg_arch_supported %s \n", __FUNCTION__, __FUNCTION__);
+
      if (!pkg_arch_supported(conf, pkg)) {
 	  ipkg_message(conf, IPKG_ERROR, "INTERNAL ERROR: architecture %s for pkg %s is unsupported.\n",
 		       pkg->architecture, pkg->name);
@@ -844,8 +857,7 @@ int ipkg_install_pkg(ipkg_conf_t *conf, pkg_t *pkg)
 	  if (ab_pkg)
 	       ab_pkg->state_status = pkg->state_status;
 
-	  ipkg_message(conf, IPKG_INFO,
-		       "Done.\n");
+	  ipkg_message(conf, IPKG_INFO, "Done.\n");
 
 	  if (use_signal)
 	       signal(SIGINT, old_handler);
@@ -1102,9 +1114,15 @@ static int check_data_file_clashes(ipkg_conf_t *conf, pkg_t *pkg, pkg_t *old_pkg
 	       /* Pre-existing files are OK if owned by a package replaced by new pkg. */
 	       if (owner) {
 		    if (pkg_replaces(pkg, owner)) {
-			 ipkg_message(conf, IPKG_INFO, "Replacing pre-existing file owned by package %s\n", filename, owner->name);
+			 ipkg_message(conf, IPKG_INFO, "Replacing pre-existing file %s owned by package %s\n", filename, owner->name);
 			 continue;
 		    }
+/* If the file that would be installed is owned by the same package, ( as per a reinstall or similar )
+   then it's ok to overwrite. */
+                    if (strcmp(owner->name,pkg->name)==0){
+			 ipkg_message(conf, IPKG_INFO, "Replacing pre-existing file %s owned by package %s\n", filename, owner->name);
+			 continue;
+                    }
 	       }
 
 	       /* Pre-existing files are OK if they are obsolete */
@@ -1202,7 +1220,7 @@ static int remove_obsolesced_files(ipkg_conf_t *conf, pkg_t *pkg, pkg_t *old_pkg
 	  if (!conf->noaction) {
 	       err = unlink(old);
 	       if (err) {
-		    ipkg_message(conf, IPKG_DEBUG, "    Warning: remove %s failed: %s\n", old,
+		    ipkg_message(conf, IPKG_ERROR, "    Warning: remove %s failed: %s\n", old,
 				 strerror(errno));
 	       }
 	  }
@@ -1335,6 +1353,9 @@ static int resolve_conffiles(ipkg_conf_t *conf, pkg_t *pkg)
      conffile_list_elt_t *iter;
      conffile_t *cf;
      char *cf_backup;
+
+    char *md5sum;
+
     
      if (conf->noaction) return 0;
 
@@ -1355,14 +1376,20 @@ static int resolve_conffiles(ipkg_conf_t *conf, pkg_t *pkg)
 
 	  cf_backup = backup_filename_alloc(root_filename);
 
+
 	  if (file_exists(cf_backup)) {
-	       if (conf->force_defaults
-		   || user_prefers_old_conffile(cf->name, cf_backup)) {
-		    rename(cf_backup, root_filename);
+ /* Let's compute md5 to test if files are changed */
+	      md5sum = file_md5sum_alloc(cf_backup);
+               if (strcmp( cf->value,md5sum) != 0 ) {
+	          if (conf->force_defaults
+		      || user_prefers_old_conffile(cf->name, cf_backup) ) {
+		       rename(cf_backup, root_filename);
+	          }
 	       }
 	       unlink(cf_backup);
 	  }
 
+	  free(md5sum);
 	  free(cf_backup);
 	  free(root_filename);
      }
